@@ -23,14 +23,21 @@
 OIDN_NAMESPACE_USING
 using namespace oidn;
 
-int numRuns = -1;
+int width  = -1;
+int height = -1;
+Format dataType = Format::Float;
+int numRuns = 4;
 int maxMemoryMB = -1;
+bool inplace = false;
 
 void printUsage()
 {
   std::cout << "Intel(R) Open Image Denoise - Benchmark" << std::endl;
-  std::cout << "usage: oidnBenchmark [-r/--run regex] [-n times]" << std::endl
-            << "                     [--threads n] [--affinity 0|1] [--maxmem MB]" << std::endl
+  std::cout << "usage: oidnBenchmark [-d/--device default|cpu]" << std::endl
+            << "                     [-r/--run regex] [-n times]" << std::endl
+            << "                     [-s/--size width height]" << std::endl
+            << "                     [-t/--type float|half]" << std::endl
+            << "                     [--threads n] [--affinity 0|1] [--maxmem MB] [--inplace]" << std::endl
             << "                     [-v/--verbose 0-3]" << std::endl
             << "                     [-l/--list] [-h/--help]" << std::endl;
 }
@@ -83,7 +90,7 @@ void addBenchmark(const std::string& filter, const std::vector<std::string>& inp
 void initImage(ImageBuffer& image, Random& rng, float minValue, float maxValue)
 {
   for (size_t i = 0; i < image.size(); ++i)
-    image[i] = minValue + rng.get1f() * (maxValue - minValue);
+    image.set(i, minValue + rng.get1f() * (maxValue - minValue));
 }
 
 // Runs a benchmark
@@ -95,40 +102,46 @@ void runBenchmark(DeviceRef& device, const Benchmark& bench)
   FilterRef filter = device.newFilter(bench.filter.c_str());
   Random rng;
 
-  ImageBuffer color;
+  std::shared_ptr<ImageBuffer> input;
+
+  std::shared_ptr<ImageBuffer> albedo;
+  if (bench.hasInput("alb"))
+  {
+    input = albedo = std::make_shared<ImageBuffer>(device, bench.width, bench.height, 3, dataType);
+    initImage(*albedo, rng, 0.f, 1.f);
+    filter.setImage("albedo", albedo->data(), albedo->format(), bench.width, bench.height);
+  }
+
+  std::shared_ptr<ImageBuffer> normal;
+  if (bench.hasInput("nrm"))
+  {
+    input = normal = std::make_shared<ImageBuffer>(device, bench.width, bench.height, 3, dataType);
+    initImage(*normal, rng, -1.f, 1.f);
+    filter.setImage("normal", normal->data(), normal->format(), bench.width, bench.height);
+  }
+
+  std::shared_ptr<ImageBuffer> color;
   if (bench.hasInput("hdr"))
   {
-    color = ImageBuffer(bench.width, bench.height, 3);
-    initImage(color, rng, 0.f, 100.f);
-    filter.setImage("color", color.data(), Format::Float3, bench.width, bench.height);
+    input = color = std::make_shared<ImageBuffer>(device, bench.width, bench.height, 3, dataType);
+    initImage(*color, rng, 0.f, 100.f);
+    filter.setImage("color", color->data(), color->format(), bench.width, bench.height);
     filter.set("hdr", true);
   }
   else if (bench.hasInput("ldr"))
   {
-    color = ImageBuffer(bench.width, bench.height, 3);
-    initImage(color, rng, 0.f, 1.f);
-    filter.setImage("color", color.data(), Format::Float3, bench.width, bench.height);
+    input = color = std::make_shared<ImageBuffer>(device, bench.width, bench.height, 3, dataType);
+    initImage(*color, rng, 0.f, 1.f);
+    filter.setImage("color", color->data(), color->format(), bench.width, bench.height);
     filter.set("hdr", false);
   }
 
-  ImageBuffer albedo;
-  if (bench.hasInput("alb"))
-  {
-    albedo = ImageBuffer(bench.width, bench.height, 3);
-    initImage(albedo, rng, 0.f, 1.f);
-    filter.setImage("albedo", albedo.data(), Format::Float3, bench.width, bench.height);
-  }
-
-  ImageBuffer normal;
-  if (bench.hasInput("nrm"))
-  {
-    normal = ImageBuffer(bench.width, bench.height, 3);
-    initImage(normal, rng, -1.f, 1.f);
-    filter.setImage("normal", normal.data(), Format::Float3, bench.width, bench.height);
-  }
-
-  ImageBuffer output(bench.width, bench.height, 3);
-  filter.setImage("output", output.data(), Format::Float3, bench.width, bench.height);
+  std::shared_ptr<ImageBuffer> output;
+  if (inplace)
+    output = input;
+  else
+    output = std::make_shared<ImageBuffer>(device, bench.width, bench.height, 3, dataType);
+  filter.setImage("output", output->data(), output->format(), bench.width, bench.height);
 
   if (maxMemoryMB >= 0)
     filter.set("maxMemoryMB", maxMemoryMB);
@@ -168,49 +181,59 @@ void runBenchmark(DeviceRef& device, const Benchmark& bench)
 // Adds all benchmarks to the list
 void addAllBenchmarks()
 {
-  const std::vector<std::pair<int, int>> sizes =
-  {
-    {1920, 1080},
-    {3840, 2160},
-    {1280, 720},
-  };
+  std::vector<std::pair<int, int>> sizes;
+
+  // Filter: RT
+#if defined(OIDN_FILTER_RT)
+  if (width < 0)
+    sizes = {{1920, 1080}, {3840, 2160}, {1280, 720}};
+  else
+    sizes = {{width, height}};
 
   for (const auto& size : sizes)
   {
     addBenchmark("RT", {"hdr", "alb", "nrm"}, size);
     addBenchmark("RT", {"ldr", "alb", "nrm"}, size);
   }
+#endif
 
-  const std::vector<std::pair<int, int>> lightmapSizes =
-  {
-    {2048, 2048},
-    {4096, 4096},
-    {1024, 1024},
-  };
+  // Filter: RTLightmap
+#if defined(OIDN_FILTER_RTLIGHTMAP)
+  if (width < 0)
+    sizes = {{2048, 2048}, {4096, 4096}, {1024, 1024}};
 
-  for (const auto& size : lightmapSizes)
+  for (const auto& size : sizes)
   {
     addBenchmark("RTLightmap", {"hdr"}, size);
   }
+#endif
 }
 
 int main(int argc, char* argv[])
 {
+  DeviceType deviceType = DeviceType::Default;
   std::string run = ".*";
   int numThreads = -1;
   int setAffinity = -1;
   int verbose = -1;
 
-  // Add the benchmarks to the list
-  addAllBenchmarks();
-
   try
   {
     ArgParser args(argc, argv);
     while (args.hasNext())
     {
       std::string opt = args.getNextOpt();
-      if (opt == "r" || opt == "run")
+      if (opt == "d" || opt == "dev" || opt == "device")
+      {
+        const auto val = args.getNextValue();
+        if (val == "default" || val == "Default")
+          deviceType = DeviceType::Default;
+        else if (val == "cpu" || val == "CPU")
+          deviceType = DeviceType::CPU;
+        else
+          throw std::invalid_argument("invalid device");
+      }
+      else if (opt == "r" || opt == "run")
         run = args.getNextValue();
       else if (opt == "n")
       {
@@ -218,21 +241,35 @@ int main(int argc, char* argv[])
         if (numRuns <= 0)
           throw std::runtime_error("invalid number of runs");
       }
+      else if (opt == "s" || opt == "size")
+      {
+        width  = args.getNextValueInt();
+        height = args.getNextValueInt();
+        if (width < 1 || height < 1)
+          throw std::runtime_error("invalid image size");
+      }
+      else if (opt == "t" || opt == "type")
+      {
+        const auto val = args.getNextValue();
+        if (val == "f" || val == "float" || val == "Float" || val == "fp32")
+          dataType = Format::Float;
+        else if (val == "h" || val == "half" || val == "Half" || val == "fp16")
+          dataType = Format::Half;
+        else
+          throw std::runtime_error("invalid data type");
+      }
       else if (opt == "threads")
         numThreads = args.getNextValueInt();
       else if (opt == "affinity")
         setAffinity = args.getNextValueInt();
-      else if (opt == "maxmem")
+      else if (opt == "maxmem" || opt == "maxMemoryMB")
         maxMemoryMB = args.getNextValueInt();
+      else if (opt == "inplace")
+        inplace = true;
       else if (opt == "v" || opt == "verbose")
         verbose = args.getNextValueInt();
       else if (opt == "l" || opt == "list")
-      {
-        // List all benchmarks
-        for (const auto& bench : benchmarks)
-          std::cout << bench.name << std::endl;
-        return 0;
-      }
+        run = "";
       else if (opt == "h" || opt == "help")
       {
         printUsage();
@@ -242,6 +279,17 @@ int main(int argc, char* argv[])
         throw std::invalid_argument("invalid argument");
     }
 
+    // Add the benchmarks to the list
+    addAllBenchmarks();
+
+    if (run.empty())
+    {
+      // List all benchmarks
+      for (const auto& bench : benchmarks)
+        std::cout << bench.name << std::endl;
+      return 0;
+    }
+
   #if defined(OIDN_X64)
     // Enable the FTZ and DAZ flags to maximize performance
     _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
@@ -249,7 +297,10 @@ int main(int argc, char* argv[])
   #endif
 
     // Initialize the device
-    DeviceRef device = newDevice();
+    DeviceRef device = newDevice(deviceType);
+
+    if (verbose >= 0)
+      device.set("verbose", verbose);
 
     const char* errorMessage;
     if (device.getError(errorMessage) != Error::None)
@@ -260,13 +311,10 @@ int main(int argc, char* argv[])
       device.set("numThreads", numThreads);
     if (setAffinity >= 0)
       device.set("setAffinity", bool(setAffinity));
-    if (verbose >= 0)
-      device.set("verbose", verbose);
+
     device.commit();
 
     // Run the benchmarks
-    if (numRuns < 0)
-       numRuns = device.get<int>("numThreads");
     const auto runExpr = std::regex(run);
     for (const auto& bench : benchmarks)
     {
