@@ -1,4 +1,4 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #pragma once
@@ -14,7 +14,9 @@ namespace oidn {
     size_t size;
 
     Data() : ptr(nullptr), size(0) {}
-    Data(void* ptr, size_t size) : ptr((char*)ptr), size(size) {}
+
+    template<typename T>
+    Data(T* ptr, size_t size) : ptr((char*)ptr), size(size) {}
 
     template<typename T, size_t N>
     Data(T (&array)[N]) : ptr((char*)array), size(sizeof(array)) {}
@@ -27,7 +29,7 @@ namespace oidn {
       return *this;
     }
 
-    operator bool() const
+    __forceinline operator bool() const
     {
       return ptr != nullptr;
     }
