@@ -9,6 +9,6 @@
 namespace oidn {
 
   // Parses tensors from a Tensor Archive (TZA)
-  std::map<std::string, Ref<Tensor>> parseTZA(const Ref<Device>& device, void* buffer, size_t size);
+  std::map<std::string, std::shared_ptr<Tensor>> parseTZA(const Ref<Device>& device, void* buffer, size_t size);
 
 } // namespace oidn
\ No newline at end of file
