@@ -21,7 +21,7 @@ class TransferFunction: pass
 def get_transfer_function(cfg):
   type = cfg.transfer
   if type == 'linear':
-    return None
+    return LinearTransferFunction()
   elif type == 'srgb':
     return SRGBTransferFunction()
   elif type == 'pu':
@@ -31,6 +31,17 @@ def get_transfer_function(cfg):
   else:
     error('invalid transfer function')
 
+## -----------------------------------------------------------------------------
+## Transfer function: Linear
+## -----------------------------------------------------------------------------
+
+class LinearTransferFunction(TransferFunction):
+  def forward(self, y):
+    return y
+
+  def inverse(self, x):
+    return x
+
 ## -----------------------------------------------------------------------------
 ## Transfer function: sRGB
 ## -----------------------------------------------------------------------------
